// LICENSE : MIT
"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _AllLiteral = require("./tag/AllLiteral");

var _FunctionType = require("./tag/FunctionType");

var _NameExpression = require("./tag/NameExpression");

var _NonNullableType = require("./tag/NonNullableType");

var _NullableType = require("./tag/NullableType");

var _RecordType = require("./tag/RecordType");

var _RestType = require("./tag/RestType");

var _TypeApplication = require("./tag/TypeApplication");

var _UnionType = require("./tag/UnionType");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var esprima = require("esprima");

var AssertGenerator = function () {
    function AssertGenerator() {
        _classCallCheck(this, AssertGenerator);
    }

    _createClass(AssertGenerator, null, [{
        key: "createAsserts",

        /**
         *
         * @param {Array<Object>} comments AST's comment nodes. it should be BlockComment
         * @returns {Array}
         */
        value: function createAsserts(comments) {
            if (comments == null) {
                return [];
            }
            var isNotEmpty = function isNotEmpty(tag) {
                return (typeof tag === "undefined" ? "undefined" : _typeof(tag)) !== undefined;
            };
            // primitive
            return comments.tags.map(AssertGenerator.createAssertFromTag).filter(isNotEmpty).map(AssertGenerator.unwrappedAST);
        }
    }, {
        key: "unwrappedAST",
        value: function unwrappedAST(string) {
            return esprima.parse(string).body[0];
        }

        /**
         * @param tagNode tagNode is defined by doctorin
         * @return {string} return assertion code string
         * Reference https://esdoc.org/tags.html#type-syntax
         * https://github.com/eslint/doctrine/blob/master/test/parse.js
         */

    }, {
        key: "createAssertFromTag",
        value: function createAssertFromTag(tagNode) {
            var tagType = tagNode.type.type;
            if (tagType === "NameExpression") {
                return (0, _NameExpression.NameExpression)(tagNode);
            } else if (tagType === "AllLiteral") {
                return (0, _AllLiteral.AllLiteral)(tagNode);
            } else if (tagType === "FunctionType") {
                return (0, _FunctionType.FunctionType)(tagNode);
            } else if (tagType === "RecordType") {
                return (0, _RecordType.RecordType)(tagNode);
            } else if (tagType === "UnionType") {
                return (0, _UnionType.UnionType)(tagNode);
            } else if (tagType === "TypeApplication") {
                return (0, _TypeApplication.TypeApplication)(tagNode);
            } else if (tagType === "RestType") {
                return (0, _RestType.RestType)(tagNode);
            } else if (tagType === "NullableType") {
                return (0, _NullableType.NullableType)(tagNode);
            } else if (tagType === "NonNullableType") {
                return (0, _NonNullableType.NonNullableType)(tagNode);
            }
        }
    }]);

    return AssertGenerator;
}();

exports.default = AssertGenerator;
module.exports = exports['default'];
//# sourceMappingURL=AssertGenerator.js.map